namespace Anue {
  namespace Auth {
    type SupportedProviderType = 'social' | 'email';
    type SupportedChannel =
      | 'stock'
      | 'mobile-app'
      | 'crypto'
      | 'driver'
      | 'unknown';

    type RefreshTokenResponse = APIResponse.Response<{
      identityId: string;
      token: string;
      newcomer?: number;
    }>;

    interface UserProfile {
      avatar: string;
      name: string;
      email: string;
      uid: string;
    }

    interface GetProfileArgs {
      token: string;
      identityId: string;
      channel: string;
    }

    interface Config {
      type?: SupportedProviderType;
      channel?: SupportedChannel;
      host?: string;
    }

    interface StateChangeEvent {
      type: 'online' | 'offline';
      context?: Credentials;
      source: string;
    }

    interface ErrorChangeEvent {
      detail: any;
      source: string;
    }

    interface PortalEvent {
      type: 'init' | 'failed' | 'success';
      payload: {
        message: string;
        status: number;
        token?: string;
        idenityId?: string;
      };
    }

    interface Credentials {
      authorization: string | undefined;
      accessToken: string | undefined;
      idToken: string | undefined;
      cognitoId: string | undefined;
      isNewComer: boolean;
      profile: UserProfile | null;
      ssoProvider: string;
    }

    interface ProviderToken {
      // This only required when `type` is `google`, also the token type header should be prepended
      idToken?: string;
      // Access token from provider, token type should also be prepended
      accessToken: string;
      oneTime?: boolean;
      renewer?(): Promise<Anue.Auth.RefreshTokenResponse>;
    }

    interface AuthContext {
      host: string;
      channel: SupportedChannel;
    }
  }
}
