declare namespace Anue {
  namespace Bridge {
    namespace URI {
      type AuthPushLogin = 'auth.push.login';
      type AuthPushLogout = 'auth.push.logout';
    }
    interface Payload<T = any> {
      rid?: number;
      uri: string;
      body: T;
    }
    type URI = URI.AuthPushLogin | URI.AuthPushLogout;

    interface PushAuthLogin {
      name: string;
      ctx: Anue.Provider.NullableResponse;
    }

    interface PushAuthLogout {
      name: string;
    }

    export interface ClientConfig {
      target: HTMLIFrameElement | Window;
    }

    export interface ServerConfig {
      target: Window;
    }

    export type Responder = (data: any) => void;
    export type Handler = (req: RequestOptions, responder: Responder) => void;
    export type RequestOptions =
      | Request.Error
      | Request.Login
      | Request.Handshake
      | Request.Push;

    export interface ServerPushOptions {
      uri: any;
    }

    export type ResponseList =
      | Response.Handshake
      | Response.Error
      | Response.Login;

    export interface Payload {
      rid: number;
      uri: string;
      body: ResponseList;
    }

    namespace Push {}

    namespace Response {
      interface Handshake {
        api: string[];
      }
      interface Login {
        token: string;
        newcomer: string;
        identifyId: string;
      }
      interface Error {
        message: string;
      }
    }

    namespace Request {
      interface Base {
        rid: number;
      }

      export interface Error extends Base {
        // rid: number
        uri: 'common.error';
        body: { message: string };
      }

      export interface Handshake extends Base {
        // rid: number
        uri: 'common.handshake';
      }

      export interface Login extends Base {
        // rid: number
        uri: 'auth.login';
        body: {
          type: string;
          idToken?: string;
          accessToken: string;
        };
      }
    }
  }
}
