namespace Anue {
  namespace Network {
    type ResponseInterceptor = (
      requestOptions: Network.RequestOptions,
      response: any
    ) => Promise<any>;

    type RequestInterceptor = (
      requestOptions: Network.RequestOptions
    ) => Promise<Network.RequestOptions>;

    interface RequestOptions<T = any> {
      method?: Network.RequestMethod;
      url: string;
      headers?: Record<string, string> | undefined;
      /**
       * @doc
       * @property body
       * 網路請求的內容，依據實作不同可以有不同的類型
       */
      body?: any;
      /**
       * @doc
       * @property extra
       * 額外的參數可以放在這裡，主要是提供給`intercepter`使用
       */
      extra?: any;
      /**
       * handles必須是一個包含一個至多個額外的ResponseInterceptor的陣列，
       * 和interceptor不同的是，這些函數只會套用到單一的請求之中，並且不影響請求的結果。
       * @doc
       * @property handles?
       */
      handles?: Array<(req: Network.RequestOptions<T>, res: T) => any>;
    }

    interface Response<T = any> {
      statusCode: number;
      headers: Record<string, string> | undefined | Headers;
      body: T | null;
      error: any;
    }
  }
}
