type ProviderResponse = Anue.Provider.NullableResponse;
type ChangeHandler = Anue.Provider.ChangeHandler;
type EventyHandler<T = any> = (e: T) => void;
type EventyRemover = () => void;

declare module 'anue-fe-sdk/obfuscator' {
  export const obfuscate = (input: string) => string;
  export const defuscate = (input: string) => string;
}

declare module 'anue-fe-sdk/getty' {
  function getty<T>(obj: any, path: Array<string | number>): T;
  export = getty;
}
declare module 'anue-fe-sdk/queryThief' {
  const queryThief = {
    get: (key: string) => string
  };

  export = queryThief;
}

declare module 'anue-fe-sdk/eventy' {
  class Eventy {
    handlers = {};
    clear(): void;
    emit(eventName: string | number, data?: any): void;
    on(eventName: string | number, fn: EventyHandler): EventyRemover;
  }
  export = Eventy;
}

declare module 'anue-fe-sdk/AbstractProvider' {
  abstract class AbstractProvider {
    protected mStateChangeHandles: ChangeHandler[];
    protected mContext: Anue.Provider.Response | null;
    abstract loginImpl(): Promise<ProviderResponse | null>;
    abstract logoutImpl(): Promise<boolean>;
    abstract getName(): string;
    abstract refresh(): Promise<ProviderResponse>;
    abstract initImpl(): Promise<AbstractProvider>;
    abstract getContext(): ProviderResponse | null;
    init: () => Promise<AbstractProvider>;
    isLoggedIn: () => boolean;
    login(args?: any): Promise<ProviderResponse | null>;
    logout: () => Promise<boolean>;
    onLoginStateChange: (fn: Anue.Provider.ChangeHandler) => AbstractProvider;
  }

  export = AbstractProvider;
}

declare module 'anue-fe-sdk/AnueMailProvider' {
  import AbstractProvider from 'anue-fe-sdk/AbstractProvider';
  export interface IMailAndPassword {
    email: string;
    password: string;
  }
  class AnueMailProvider extends AbstractProvider {
    constructor();
    init(): Promise<GoogleProvider>;
    login(data: IMailAndPassword): Promise<Anue.Provider.Response>;
    isLogin(): boolean;
    getContext(): Anue.Provider.NullableResponse;
    logout(): Promise<boolean>;
    refresh(): Promise<Anue.Provider.Response>;
  }
  export = AnueMailProvider;
}

declare module 'anue-fe-sdk/FacebookWebProvider' {
  import AbstractProvider from 'anue-fe-sdk/AbstractProvider';

  class FacebookWebProvider extends AbstractProvider {
    constructor(config: { appId: string });
    init(): Promise<GoogleProvider>;
    login(): Promise<Anue.Provider.Response>;
    isLogin(): boolean;
    getContext(): Anue.Provider.NullableResponse;
    logout(): Promise<boolean>;
    refresh(): Promise<Anue.Provider.Response>;
  }

  export = FacebookWebProvider;
}

declare module 'anue-fe-sdk/GoogleWebProvider' {
  import AbstractProvider from 'anue-fe-sdk/AbstractProvider';
  export default class GoogleProvider extends AbstractProvider {
    private mIsScriptLoaded;
    private loginPromises;
    private logoutPromises;
    private mAuth;
    private mAppId;
    private mApiKey;
    constructor(config: Anue.Provider.GoogleProviderConfig);
    private getCurrentUser;
    getName: () => string;
    getType: () => string;
    getContext: () => Anue.Provider.NullableResponse;
    refresh: () => Promise<Anue.Provider.Response>;
    private populatePromises;
    handleLoginState: (loggedIn: boolean) => void;
    initImpl: () => Promise<AbstractProvider>;
    logoutImpl: () => Promise<boolean>;
    loginImpl: () => Promise<Anue.Provider.NullableResponse>;
  }

  export = GoogleProvider;
}

declare module 'anue-fe-sdk/Constants' {
  export enum SupportedProviderType {
    Email = 'email',
    Social = 'social'
  }

  export enum SupportedChannel {
    Stock = 'stock',
    MobileApp = 'mobile-app',
    Crypto = 'crypto',
    Driver = 'driver',
    Unknown = 'unknown'
  }
}

declare module 'anue-fe-sdk/APIs' {
  namespace Email {
    export function verifyActivationCode(checkCode: string): Promise<boolean>;
    export function register(
      name: string,
      email: string,
      password: string,
      channel: string
    ): Promise<number>;
    export function resend(email: string): Promise<number>;
  }

  namespace Profile {
    export function updatePassword(
      oldPassword: string,
      newPassword: string
    ): Promise<number>;
    export function recoverPassword(email: string): Promise<number>;
    export function resetPassword(
      recoverKey: string,
      password: string
    ): Promise<number>;
  }
}

declare module 'anue-fe-sdk/web-bridge-server' {
  interface ServerConfig {
    target: Window;
  }

  class BridgeServer {
    constructor(config: ServerConfig);
    start(): Promise<boolean>;
    use(uri: string, responder: any);
    push(uri: string, payload: any);
  }
  export = BridgeServer;
}

declare module 'anue-fe-sdk/Auth' {
  import Constants from 'anue-fe-sdk/Constants';
  import AbstractProvider from 'anue-fe-sdk/AbstractProvider';
  import Eventy from 'anue-fe-sdk/eventy';

  class Auth {
    event: Eventy;
    channel(
      channel: 'stock' | 'mobile-app' | 'crypto' | 'driver' | 'unknown'
    ): Auth;
    accessToken(token: string): Auth;
    portal(url: string): Auth;
    launchPortal(): Auth;
    renew(): Promise<Anue.Auth.Credentials | null>;
    ready(): Promise<Anue.Auth.Credentials | null>;
    getCredentialContext(): Anue.Auth.Credentials | null;
    connectProvider(...provider: AbstractProvider[]): Auth;
    lazyConnectProvider(...provider: AbstractProvider[]): Auth;
    getPrimaryProvider(): AbstractProvider;
    restore(): Promise<boolean>;
    onStateChange(fn: EventyHandler<Anue.Auth.StateChangeEvent>): Auth;
    clearCredentials();
  }
  export = Auth;
}
