import Bridge = Anue.Bridge;

export default abstract class AbstractServerBridge {
  static NotInitializedError = {
    rid: -1,
    body: {
      uri: '/common/error',
      message: 'Anue.Bridge not initialized'
    }
  };

  protected mConfig: Anue.Bridge.ServerConfig | null = null;
  protected mApiTable: Record<string, Bridge.Handler> = {};
  protected mWindow: Window | undefined;

  protected sendResponse = (
    req: Bridge.RequestOptions | Bridge.ServerPushOptions
  ) => (response: any) => {
    if (this.mWindow) {
      this.mWindow.postMessage(
        {
          rid: req.rid,
          uri: req.uri,
          body: response
        },
        '*'
      );
    }
  };

  protected listen = (message: MessageEvent) => {
    try {
      const req =
        typeof message.data === 'object'
          ? message.data
          : (JSON.parse(message.data) as Anue.Bridge.RequestOptions);
      const handle = this.mApiTable[req.uri];
      handle(req, this.sendResponse(req));
    } catch (err) {
      // console.log('Anue.Bridge.Server failed to parse unknown event', err)
    }
  };

  constructor(config: Anue.Bridge.ServerConfig) {
    this.mConfig = config;
    this.mApiTable['common.handshake'] = (req, responder) => {
      const apis: string[] = [];
      for (const api in this.mApiTable) {
        apis.push(api);
      }
      responder(apis);
    };
  }

  abstract start(): Promise<boolean>;

  use = (uri: string, handler: Bridge.Handler): AbstractServerBridge => {
    this.mApiTable[uri] = handler;
    return this;
  };

  push = (uri: any, payload: any) => {
    this.sendResponse({ uri })(payload);
  };
}
