import Keys from '@constants/Keys';
import SharedContext from '@libraries/SharedContext';

let spy = jest.fn();

SharedContext.set(Keys.Debuggable.Config, {
  type: '(FUNC|SET)',
  name: 'TestImpl'
});

import Debuggable from '@libraries/Debuggable';

class TestImpl extends Debuggable {
  prop: any = null;

  test1 = args => {
    args.invoke = 1;
    return args;
  };
}

class NoLogImpl extends Debuggable {
  prop = null;

  test2 = args => {
    args.invoke = 1;
    return args;
  };
}

describe('Debuggable tests', () => {
  let obj = { date: Date.now() };
  const target = new TestImpl();

  it('logs function invocation', () => {
    let spy = jest.fn();
    SharedContext.set(Keys.Debuggable.Logger, spy);
    const actual = target.test1(obj);
    const expected = { ...obj, invoke: 1 };
    expect(actual).toEqual(expected);
    expect(spy).toBeCalledWith('FUNC', 'TestImpl', 'test1', [obj], expected);
  });

  it('log filter test, should not log GET operation', () => {
    target.prop;
    expect(spy).not.toBeCalledWith('GET', 'TestImpl', 'prop');
  });

  it('log filter test, should log SET operation', () => {
    let stack: any[] = [];
    let spy = jest.fn().mockImplementation((...args) => {
      stack.push(args);
    });
    SharedContext.set(Keys.Debuggable.Logger, spy);
    target.prop = 1;
    expect(stack).toEqual([['SET', 'TestImpl', 'prop', 1, undefined]]);
  });

  it('can filter irrelevant classes', () => {
    new NoLogImpl().test2({});
    expect(spy).not.toBeCalled();
  });
});
