import getty from '@utils/getty';
import * as Network from '@libraries/Network';

export type IRequestOptions<T> = Anue.Network.RequestOptions<T>;
export type APITransform<T> = (resp: APIResponse.Response<T>) => any;

export default class API<T> {
  private mHandles: Array<(req: IRequestOptions<T>, res: T) => any> = [];
  private mOptions: Anue.Network.RequestOptions | null = null;
  private mTransform: APITransform<T> = API.BaseTransform;

  static BaseTransform = (resp: APIResponse.Response<any>) => {
    return resp.items;
  };

  transform = (fn: APITransform<T>) => {
    this.mTransform = fn;
  };

  handle = (
    condition: number | ((req: IRequestOptions<T>, res: T) => boolean),
    handle: (req: IRequestOptions<T>, res: T) => any
  ): API<T> => {
    if (typeof condition === 'number') {
      this.mHandles.push((req, res) => {
        if (getty<number>(res, ['statusCode']) === condition) {
          return handle(req, res);
        }
      });
    } else {
      this.mHandles.push((req, res) => {
        if (condition(req, res)) {
          return handle(req, res);
        }
      });
    }

    return this;
  };

  options = (options: Anue.Network.RequestOptions) => {
    this.mOptions = options;
    return this;
  };

  send = async (): Promise<T> => {
    if (!this.mOptions) {
      throw {
        message:
          'An API call cannot be sent without any options, call .options(opts) first'
      };
    }
    this.mOptions.handles = this.mHandles;
    const response = await Network.getDriver().send<APIResponse.Response<T>>(
      this.mOptions
    );
    return this.mTransform(response);
  };
}
