/**
 * @doc
 * @anue-package Anue.API.Email
 * 和電子郵件認證相關的API實作與集合，使用方式
 *
 * ```Javascript
 * import 'anue-fe-sdk'
 * import 'anue-fe-sdk/web-preset'
 * import Email from 'anue-fe-sdk/API/Email'
 *
 * // ...
 *
 * ```
 */

import * as Network from '@libraries/Network';
import Endpoints from '@constants/Endpoints';
import Misc from '@constants/Misc';
import getty from '@utils/getty';

type EmailVerify = APIResponse.Response<
  APIResponse.v1.User.Registration.Email.Verify
>;

type EmailRegistration = APIResponse.Response<
  APIResponse.v1.User.Registration.Email
>;

type EmailVerifyResend = APIResponse.Response<APIResponse.v1.User.Email.Resend>;

export default {
  verifyActivationCode: async (checkCode: string): Promise<boolean> => {
    const resp = await Network.getDriver().send<EmailVerify>({
      method: Misc.RequestMethod.PUT,
      url: Endpoints.v1User.registration.email.verify(),
      body: { checkCode }
    });
    const code = await resp.statusCode;
    if (code !== 200) {
      throw {
        message: 'failed to verify user',
        detail: resp
      };
    }

    return getty<boolean>(resp, ['items', 'verify']) || false;
  },

  register: async (
    name: string,
    email: string,
    password: string,
    channel: string
  ): Promise<number> => {
    const resp = await Network.getDriver().send<EmailRegistration>({
      method: Misc.RequestMethod.POST,
      url: Endpoints.v1User.registration.email(),
      body: { name, email, password, channel }
    });
    const code = await resp.statusCode;
    if (code !== 200) {
      throw {
        message: 'failed to register user',
        detail: resp
      };
    }
    return code;
  },

  resend: async (email: string): Promise<number> => {
    const resp = await Network.getDriver().send<EmailVerifyResend>({
      method: Misc.RequestMethod.POST,
      url: Endpoints.v1User.email.resend(),
      body: { email }
    });
    const code = await resp.statusCode;
    if (code !== 200) {
      throw {
        message: 'failed to resend verify email',
        detail: resp
      };
    }
    return code;
  }
};
