import Endpoints from '@constants/Endpoints';
import Debuggable from '@libraries/Debuggable';
import Misc from '@constants/Misc';
import * as Network from '@libraries/Network';

class TOS extends Debuggable {
  /**
   * @doc
   * @public
   * @method getAllTOS
   * @promise
   * @returns APIResponse.v1.User.TOS.All
   * 單純顯示 ToS，無須登入
   */
  getAllTOS = async (
    service: string = 'ALL'
  ): Promise<APIResponse.v1.User.TOS.All> => {
    const endpoint = Endpoints.api.v1.user.tos.all();
    const network = Network.getDriver();
    const response = network.send({
      method: Misc.RequestMethod.GET,
      headers: {
        'content-type': 'application/json'
      },
      url: service
        ? endpoint.toString() + `?service=${service}`
        : endpoint.toString()
    });

    return await response;
  };

  /**
   * @doc
   * @public
   * @method updateTOS
   * @promise
   * @returns APIResponse.v1.User.TOS
   * 使用則接受 ToS
   */
  updateTOS = async (
    platform: string = 'DESKTOP',
    services: string = 'ALL'
  ): Promise<APIResponse.v1.User.TOS> => {
    const endpoint = Endpoints.api.v1.user.tos();
    const network = Network.getDriver();
    const response = network.send({
      method: Misc.RequestMethod.PUT,
      headers: {
        'content-type': 'application/json'
      },
      url: endpoint.toString(),
      body: JSON.stringify({
        platform,
        services
      })
    });

    return await response;
  };

  /**
   * @doc
   * @public
   * @method checkUserTOS
   * @promise
   * @returns APIResponse.v1.User.TOS.User
   * 使用者登入後，檢查 ToS 狀態
   */
  checkUserTOS = async (
    service: string = 'ALL'
  ): Promise<APIResponse.v1.User.TOS.User> => {
    const endpoint = Endpoints.api.v1.user.tos.user();
    const network = Network.getDriver();
    const response = network.send({
      method: Misc.RequestMethod.GET,
      headers: {
        'content-type': 'application/json'
      },
      url: service
        ? endpoint.toString() + `?service=${service}`
        : endpoint.toString()
    });

    return await response;
  };
}

export default TOS;
