import Endpoints from '@constants/Endpoints';
import Auth from '@packages/Auth';
import NetworkMockDriver from '@mocks/mock-network';
import StorageMockDriver from '@mocks/mock-storage';
import * as Network from '@libraries/Network';
import * as Storage from '@libraries/Storage';
import { SupportedChannel } from '@constants/Keys';
import MockProvider, {
  mockContext
} from '@packages/Auth/__mocks__/MockProvider.mock';
import urly from '@utils/urly';
import waitty from '@utils/waitty';
import Shared from '@libraries/SharedContext';
// import getty from '@utils/getty';

const host = 'mock://auth.api';
const auth = new Auth().host(host).channel(SupportedChannel.Driver);

beforeAll(() => {
  Network.setDriver(NetworkMockDriver);
  Storage.setDriver(StorageMockDriver);
});

function mockRenewSuccess() {
  // set request HTTP mock
  const tester = opt =>
    opt.url.includes(
      urly(host, Endpoints.api.v1.user.cognitoOpenIdOrRegister())
    );
  const responder = () => ({
    items: {
      identityId: '$identityId',
      token: '$token',
      newcomer: 1
    },
    message: '成功',
    statusCode: 200
  });
  return NetworkMockDriver.mock(tester, responder);
}

describe('General Auth tests', () => {
  const mockProviders = [1, 2, 3].map(n => new MockProvider(n));
  mockRenewSuccess();

  it('connect providers should also initialize them, and ready will guarantee they been initialized', async () => {
    await auth.connectProvider(...mockProviders).ready();
    mockProviders.forEach((m, n) => {
      expect(m.getContext()).toEqual(mockContext(n + 1));
    });
  });

  it('state change event should be triggered when provider login/logout', async () => {
    const tar = mockProviders[2].getName();
    const actual: string[] = [];
    const expected: string[] = [tar];
    const listener = jest.fn().mockImplementation(e => {
      actual.push(e.source);
      expect(actual).toEqual(expected);
    });
    auth.onStateChange(listener);
    auth.setSSOProvider(tar);
    await auth.ready();

    await mockProviders[2].login();
    await mockProviders[0].login();
    await mockProviders[1].login();
    await waitty(1000);
    expect(listener).toBeCalledTimes(1);
    return;
  }, 2000);

  it('sso provider state is consumed correctly, non-sso provider will not be initialized', async () => {
    const state = Date.now();
    const providers = [1, 2, 3].map(n => new MockProvider(n));
    await Storage.getDriver().setItem('anue.state', state);
    Shared.set('urld', {
      state,
      provider: providers[0].getName()
    });
    const auth2 = new Auth();
    await auth2.connectProvider(...providers).ready();
    // await waitty(1)
    console.log('examing ...');
    expect(providers[0].getContext()).toEqual(mockContext(1));
    expect(providers[1].getContext()).toBe(null);
    expect(providers[2].getContext()).toBe(null);
  });
});
