import AbstractProvider from './AbstractProvider.abstract';
import Keys from '@constants/Keys';
import getty from '@utils/getty';

export default class FacebookWebProvider extends AbstractProvider {
  private mAuth;
  private scriptImported = false;
  private mAppId;

  constructor(config: { appId: string }) {
    super();
    this.mAppId = config.appId;
  }

  private transform = (data): Anue.Provider.NullableResponse => {
    const accessToken = getty(data, ['authResponse', 'accessToken']);
    if (accessToken) {
      return { accessToken, tokenType: '' };
    }
    return null;
  };

  private handleStateChange = response => {
    console.log('[FBProvider] status changed', response);
    this.mContext = this.transform(response);
    this.emitStateChange();
  };

  loginImpl = (): Promise<Anue.Provider.NullableResponse> =>
    new Promise((resolve, reject) => {
      this.mAuth.getLoginStatus(res => {
        this.mAuth.login(
          response => {
            console.log('[FB] login response %o', response);
            if (response.status === 'connected') {
              resolve(this.transform(response));
            } else {
              reject(response);
            }
          },
          { scope: 'email' }
        );
      });
    });

  logoutImpl = (): Promise<boolean> =>
    new Promise(resolve => {
      // @ts-ignore
      FB.logout(response => {
        console.log('[FB] logout response %o', response);
        resolve();
      });
    });

  getName = () => 'facebook';

  getType = () => Keys.SupportedProviderType.Social;

  refresh(): Promise<Anue.Provider.NullableResponse> {
    throw new Error('Method not implemented.');
  }

  initImpl = (): Promise<AbstractProvider> => {
    // @ts-ignore
    if (typeof document === 'undefined' || this.mAuth || this.scriptImported) {
      return Promise.resolve(this);
    }
    this.scriptImported = true;

    return new Promise(async resolve => {
      // tslint:disable
      (function(d, s, id) {
        var js,
          fjs = d.getElementsByTagName(s)[0];
        if (d.getElementById(id)) {
          return;
        }
        js = d.createElement(s);
        js.id = id;
        js.src = 'https://connect.facebook.net/en_US/sdk.js';
        // @ts-ignore
        fjs.parentNode.insertBefore(js, fjs);
      })(document, 'script', 'facebook-jssdk');
      // @ts-ignore
      window.fbAsyncInit = () => {
        // @ts-ignore
        this.mAuth = FB;
        this.mAuth.Event.subscribe(
          'auth.authResponseChange',
          this.handleStateChange
        );
        // @ts-ignore
        FB.init({
          appId: this.mAppId,
          autoLogAppEvents: true,
          xfbml: true,
          version: 'v3.2'
        });
        this.mAuth.getLoginStatus();
        resolve(this);
        // tslint:enable
      };
    });
  };

  getContext(): Anue.Provider.NullableResponse {
    return this.mContext;
  }
}
