import axios, { AxiosResponse } from 'axios';
import AbstractNetworkInterface from '@libraries/Network/Network.abstract';

class AxiosDriverNetworkDriver extends AbstractNetworkInterface {
  private instance = axios.create();

  /**
   * Send a request and resolves the response, this only throws error
   * when things going wrong with network connection.
   * otherwise it resolves an Anue.Response object with status code,
   * body and other information.
   * @param options Request options like url, method, headers, and body
   */
  request = async <T = any>(
    options: Anue.Network.RequestOptions
  ): Promise<T> => {
    if (options && typeof options.body === 'object') {
      options.headers = options.headers || {};
      options.headers['content-type'] = 'application/json';
      options.body = JSON.stringify(options.body);
    }

    const res: AxiosResponse = await this.instance.request({
      url: options.url,
      method: options.method,
      headers: options.headers,
      data: options.body
    });

    return await res.data;
  };
}

// Provide aliases for supported request methods
['delete', 'get', 'head', 'options'].forEach(function forEachMethodNoData(
  method
) {
  AxiosDriverNetworkDriver.prototype[method] = function(url, config) {
    return this.send({
      ...config,
      method,
      url
    });
  };
});

['post', 'put', 'patch'].forEach(function forEachMethodWithData(method) {
  AxiosDriverNetworkDriver.prototype[method] = function(url, data, config) {
    return this.send({
      ...config,
      method,
      url,
      data
    });
  };
});

export default AxiosDriverNetworkDriver;
