const path = require('path')
const nodeExternals = require('webpack-node-externals');

module.exports = {
  mode: 'development',
  target: 'node',
  entry: path.resolve(__dirname, './sandbox/server.js'),
  module: {
    rules: [{
      test: /\.js$/,
      use: {
        loader: 'babel-loader',
        options: {
          presets: ['@babel/preset-env', '@babel/preset-react'],
          plugins: ["@babel/plugin-proposal-class-properties"]
        }
      },
      exclude: /node_modules/
    }, {
      test: /\.scss$/,
      use: [
        "isomorphic-style-loader", // creates style nodes from JS strings
        {
          loader: 'css-loader',
          options: {
            importLoaders: 1
          }
        },
        "postcss-loader" // compiles Sass to CSS, using Node Sass by default
      ]
    }]
  },
  resolve: {
    extensions: ['.js'],
    alias: {
      'anue-fe-sdk': path.resolve(__dirname, './build')
    }
  },
  output: {
    filename: "server.bundle.js",
    path: path.resolve(__dirname, './sandbox'),
    umdNamedDefine: true,
    globalObject: 'typeof self !== \'undefined\' ? self : this',
  },
  externals: [nodeExternals()]
}
